package com.ruoyi.system.service.impl;

import com.ruoyi.common.core.exception.ServiceException;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.common.redis.service.RedisService;
import com.ruoyi.system.domain.SysConfig;
import com.ruoyi.system.mapper.SysConfigMapper;
import com.ruoyi.system.service.ISysConfigService;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import javax.annotation.PostConstruct;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class SysConfigServiceImpl implements ISysConfigService {
   @Autowired
   private SysConfigMapper configMapper;
   @Autowired
   private RedisService redisService;

   @PostConstruct
   public void init() {
      this.loadingConfigCache();
   }

   public SysConfig selectConfigById(Long configId) {
      SysConfig config = new SysConfig();
      config.setConfigId(configId);
      return this.configMapper.selectConfig(config);
   }

   public String selectConfigByKey(String configKey) {
      String configValue = Convert.toStr(this.redisService.getCacheObject(this.getCacheKey(configKey)));
      if (StringUtils.isNotEmpty(configValue)) {
         return configValue;
      } else {
         SysConfig config = new SysConfig();
         config.setConfigKey(configKey);
         SysConfig retConfig = this.configMapper.selectConfig(config);
         if (StringUtils.isNotNull(retConfig)) {
            this.redisService.setCacheObject(this.getCacheKey(configKey), retConfig.getConfigValue());
            return retConfig.getConfigValue();
         } else {
            return "";
         }
      }
   }

   public SysConfig selectConfigByKeyObj(String configKey) {
      SysConfig config = new SysConfig();
      config.setConfigKey(configKey);
      SysConfig retConfig = this.configMapper.selectConfig(config);
      if (StringUtils.isNotNull(retConfig)) {
         this.redisService.setCacheObject(this.getCacheKey(configKey), retConfig.getConfigValue());
         return retConfig;
      } else {
         return null;
      }
   }

   public List<SysConfig> selectConfigList(SysConfig config) {
      return this.configMapper.selectConfigList(config);
   }

   public int insertConfig(SysConfig config) {
      int row = this.configMapper.insertConfig(config);
      if (row > 0) {
         this.redisService.setCacheObject(this.getCacheKey(config.getConfigKey()), config.getConfigValue());
      }

      return row;
   }

   public int updateConfig(SysConfig config) {
      SysConfig temp = this.configMapper.selectConfigById(config.getConfigId());
      if (!StringUtils.equals(temp.getConfigKey(), config.getConfigKey())) {
         this.redisService.deleteObject(this.getCacheKey(temp.getConfigKey()));
      }

      int row = this.configMapper.updateConfig(config);
      if (row > 0) {
         this.redisService.setCacheObject(this.getCacheKey(config.getConfigKey()), config.getConfigValue());
      }

      return row;
   }

   public void deleteConfigByIds(Long[] configIds) {
      Long[] var2 = configIds;
      int var3 = configIds.length;

      for(int var4 = 0; var4 < var3; ++var4) {
         Long configId = var2[var4];
         SysConfig config = this.selectConfigById(configId);
         if (StringUtils.equals("Y", config.getConfigType())) {
            throw new ServiceException(String.format("内置参数【%1$s】不能删除 ", config.getConfigKey()));
         }

         this.configMapper.deleteConfigById(configId);
         this.redisService.deleteObject(this.getCacheKey(config.getConfigKey()));
      }

   }

   public void loadingConfigCache() {
      List<SysConfig> configsList = this.configMapper.selectConfigList(new SysConfig());
      Iterator var2 = configsList.iterator();

      while(var2.hasNext()) {
         SysConfig config = (SysConfig)var2.next();
         this.redisService.setCacheObject(this.getCacheKey(config.getConfigKey()), config.getConfigValue());
      }

   }

   public void clearConfigCache() {
      Collection<String> keys = this.redisService.keys("sys_config:*");
      this.redisService.deleteObject(keys);
   }

   public void resetConfigCache() {
      this.clearConfigCache();
      this.loadingConfigCache();
   }

   public boolean checkConfigKeyUnique(SysConfig config) {
      Long configId = StringUtils.isNull(config.getConfigId()) ? -1L : config.getConfigId();
      SysConfig info = this.configMapper.checkConfigKeyUnique(config.getConfigKey());
      return !StringUtils.isNotNull(info) || info.getConfigId() == configId;
   }

   private String getCacheKey(String configKey) {
      return "sys_config:" + configKey;
   }
}
